---------------------------------------------------------------------------------------------------------------
-- RISA Project
-- Author: A.Greensted
-- Module:
-- Description:
-- History:
---------------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package CommonPkg is

	-----------------------------------------------------------------------------------------
	-- Constants
	-----------------------------------------------------------------------------------------
	constant uns_0000						: unsigned				:= b"0000_0000_0000_0000";
	constant slv_0000						: std_logic_vector	:= b"0000_0000_0000_0000";

	constant uns_0001						: unsigned				:= b"0000_0000_0000_0001";
	constant slv_0001						: std_logic_vector	:= b"0000_0000_0000_0001";

	-----------------------------------------------------------------------------------------
	-- Type Conversion Functions
	-----------------------------------------------------------------------------------------
	function toChar( sl : std_logic) return character;
	function toString( slv : std_logic_vector) return string;

	-----------------------------------------------------------------------------------------
	-- Components
	-----------------------------------------------------------------------------------------
	component PulseGenerator is
		port (	clk					: in	std_logic;
					nReset				: in	std_logic;
					enable				: in	std_logic;
					input					: in	std_logic;
					pulse					: out	std_logic);
	end component;
	
	component Synchroniser is
		port (	clk					: in		std_logic;
					dIn					: in		std_logic;
					dOut					: out		std_logic);
	end component;

	component EdgeSynchroniser is
		port (	clk					: in		std_logic;
					dIn					: in		std_logic;
					dOut					: out		std_logic);
	end component;

	component PulseSynchroniser is
		port (	clk					: in		std_logic;
					dIn					: in		std_logic;
					dOut					: out		std_logic);
	end component;

	component RandomNumberGenerator is
		port (	clk					: in	std_logic;
					enable				: in	std_logic;
					nReset				: in	std_logic;
					dOut					: out	std_logic_vector(15 downto 0));
	end component;

	component Fifo is
		generic(	dataWidth			: in		positive := 8);
		port (	clk					: in		std_logic;
					nReset				: in		std_logic;
					enable				: in		std_logic;
					wEn					: in		std_logic;
					rEn					: in		std_logic;
					dIn					: in		std_logic_vector(dataWidth-1 downto 0);
					dOut					: out		std_logic_vector(dataWidth-1 downto 0);
					full					: out		std_logic;
					empty					: out		std_logic;
					fillLevel			: out		std_logic_vector(1 downto 0));
	end component;

	component RAM4Kx16_Sim is
		port (	clk					: in	std_logic;
					wEn					: in	std_logic;
					enable				: in	std_logic;
					add					: in	std_logic_vector(11 downto 0);
					dIn					: in	std_logic_vector(15 downto 0);
					dOut					: out	std_logic_vector(15 downto 0));
	end component;

	component Memory is
		port (	clk					: in		std_logic;
					enable				: in		std_logic;
					nReset				: in		std_logic;
					wEn					: in		std_logic;
					add					: in		std_logic_vector(12 downto 0);
					dIn					: in		std_logic_vector(15 downto 0);
					dOutLU				: out		std_logic_vector(15 downto 0);
					dOutU					: out		std_logic_vector(15 downto 0));
	end component;

end CommonPkg;

package body CommonPkg is

function toChar(sl: std_logic) return character is
	variable c: character;
begin
	case sl is
		when 'U' => c:= 'U';
		when 'X' => c:= 'X';
		when '0' => c:= '0';
		when '1' => c:= '1';
		when 'Z' => c:= 'Z';
		when 'W' => c:= 'W';
		when 'L' => c:= 'L';
		when 'H' => c:= 'H';
		when '-' => c:= '-';
	end case;
	return c;
end toChar;

function toString(slv: std_logic_vector) return string is
	variable result : string (1 to slv'length);
	variable r : integer;
begin
	r := 1;
	for i in slv'range loop
		result(r) := toChar(slv(i));
		r := r + 1;
	end loop;
	return result;
end toString;

end CommonPkg;
