---------------------------------------------------------------------------------------------------------------
-- RISA Project
-- Author: A.Greensted
-- Module: Memory (For VST - ASIC)
-- Description:
-- History:
---------------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity Memory is
	port (	clk			: in		std_logic;
				enable		: in		std_logic;
				nReset		: in		std_logic;

				wEn			: in		std_logic;
				add			: in		std_logic_vector(12 downto 0);
				dIn			: in		std_logic_vector(15 downto 0);

				dOutLU		: out		std_logic_vector(15 downto 0);
				dOutU			: out		std_logic_vector(15 downto 0));
end Memory;

architecture VST of Memory is

	-- VST RAM
	component RAM4kx16_150c is
		port (	CK				: in		std_logic;
					ADR			: in		std_logic_vector(11 downto 0);
					DI				: in		std_logic_vector(15 downto 0);
					WEN			: in		std_logic;
					CEN			: in		std_logic;
					OEN			: in		std_logic;
					DOUT			: out		std_logic_vector(15 downto 0));
	end component;

	signal ramEnable		: std_logic;

	signal wEnLower		: std_logic;
	signal wEnUpper		: std_logic;

	signal dOutUpper		: std_logic_vector(15 downto 0);
	signal dOutLower		: std_logic_vector(15 downto 0);

	signal outputSelect	: std_logic;

begin

ramEnable <= not enable;

-- Write Control ----------------------------------------------------------------------------------------------
-- RAM Write Enable signals, RAM4kx16_150 have active low WEN signals
----------------------------------------------------------------------------------------------------------------
wEnUpper	<= not (wEn and add(0));
wEnLower	<= not (wEn and not add(0));


-- RAM Blocks Instantiation -----------------------------------------------------------------------------------
-- Two RAM blocks, one for the upper 16 bits, one for the lower
----------------------------------------------------------------------------------------------------------------
RamUpper : RAM4kx16_150c
port map	(	CK				=> clk,
				ADR			=> add(12 downto 1),
				DI				=> dIn,
				WEN			=> wEnUpper,
				CEN			=> ramEnable,
				OEN			=> ramEnable,
				DOUT			=> dOutUpper);

RamLower : RAM4kx16_150c
port map	(	CK				=> clk,
				ADR			=> add(12 downto 1),
				DI				=> dIn,
				WEN			=> wEnLower,
				CEN			=> ramEnable,
				OEN			=> ramEnable,
				DOUT			=> dOutLower);

-- Output Control ---------------------------------------------------------------------------------------------
-- Output Selection
----------------------------------------------------------------------------------------------------------------
OutputSelector : process(clk)
begin

	if (clk'event and clk='1') then

		if (nReset='0') then
			outputSelect <= '0';

		elsif (enable='1') then
			outputSelect <= add(0);

		end if;

	end if;

end process;

-- Output Mux
dOutLU	<= dOutLower when (outputSelect='0') else dOutUpper;
dOutU		<= dOutUpper;

end VST;
