---------------------------------------------------------------------------------------------------------------
-- RISA Project
-- Author: A.Greensted
-- Module:
-- Description:
-- History:
---------------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package ConfigPkg is
	-----------------------------------------------------------------------------------------
	-- Types / Constants
	-----------------------------------------------------------------------------------------
	type CONFIGTYPE_CHAIN		is (	CONFIG_CNF_CHAIN,
												CONFIG_SEL_CHAIN);

	type CONFIGTYPE_TARGET		is (	CONFIG_LOGIC_TARGET,
												CONFIG_ROUTE_TARGET);

	type CONFIGTYPE_CAP_TARGET		is (	CONFIG_CAP_TARGET_NULL,
													CONFIG_CAP_TARGET_UP_CONTROL,
													CONFIG_CAP_TARGET_LOGIC_CNF_CHAIN,
													CONFIG_CAP_TARGET_LOGIC_SEL_CHAIN,
													CONFIG_CAP_TARGET_ROUTE_CNF_CHAIN,
													CONFIG_CAP_TARGET_ROUTE_SEL_CHAIN,
													CONFIG_CAP_TARGET_UP_MEM_DATA,
													CONFIG_CAP_TARGET_UP_MEM_ADDRESS);



	-----------------------------------------------------------------------------------------
	-- Type Conversion Functions
	-----------------------------------------------------------------------------------------
	function to_CONFIGTYPE_CHAIN( value : std_logic ) return CONFIGTYPE_CHAIN;
	function to_std_logic( value : CONFIGTYPE_CHAIN ) return std_logic;

	function to_CONFIGTYPE_TARGET( value : std_logic ) return CONFIGTYPE_TARGET;
	function to_std_logic( value : CONFIGTYPE_TARGET ) return std_logic;

	function to_CONFIGTYPE_CAP_TARGET( value : std_logic_vector(2 downto 0) ) return CONFIGTYPE_CAP_TARGET;
	function to_std_logic_vector( value : CONFIGTYPE_CAP_TARGET ) return std_logic_vector;

	-----------------------------------------------------------------------------------------
	-- Components
	-----------------------------------------------------------------------------------------
	component ConfigAccessPort_CAP is
		port (	cap_clk				: in		std_logic;
					cap_nReset			: in		std_logic;
					cap_enable			: in		std_logic;
					cap_ctrlDataSel	: in		std_logic;
					cap_shiftExecSel	: in		std_logic;
					cap_dIn				: in		std_logic;
					cap_dOut				: out		std_logic;
					cap_busy				: out		std_logic;
					start					: out		std_logic;
					release				: in		std_logic;
					controlData			: out		std_logic_vector(10 downto 0);
					sendData				: out		std_logic_vector(15 downto 0);
					returnData			: in		std_logic_vector(15 downto 0));
	end component;

	component ConfigAccessPort_CNF
		port (	clk					: in		std_logic;
					cnf_nReset			: in		std_logic;
					cnf_dIn				: in		std_logic;
					cnf_dOut				: out		std_logic;
					cnf_shiftEn			: out		std_logic;
					cnf_ctrl				: out		std_logic_vector(1 downto 0);
					cnf_target			: out		CONFIGTYPE_CAP_TARGET;
					cnf_targetSetEn	: out		std_logic_vector(3 downto 0);
					start					: in		std_logic;
					release				: out		std_logic;
					controlData			: in		std_logic_vector(10 downto 0);
					sendData				: in		std_logic_vector(15 downto 0);
					returnData			: out		std_logic_vector(15 downto 0));
	end component;

	component ConfigSelectUnit is
		port (	clk									: in		std_logic;
					nReset								: in		std_logic;
					selChnDIn							: in		std_logic;
					selChnDOut							: out		std_logic;
					selChnShiftEn						: in		std_logic;
					cnfChnDIn							: in		std_logic;
					cnfChnDOut							: out		std_logic;
					cnfChnShiftEn						: in		std_logic;
					cnfChnLoadEn						: in		std_logic;
					cnfChnReadBackEn					: in		std_logic;
					cnfChnDInBP							: in		std_logic;
					cnfChnDOutBP						: out		std_logic;
					cnfChnShiftEnBP					: out		std_logic;
					cnfChnLoadEnBP						: out		std_logic;
					cnfChnReadBackEnBP				: out		std_logic);
	end component;

	component ConfigurableBit is
		port (	clk									: in		std_logic;
					nReset								: in		std_logic;
					cnfBit								: out		std_logic;
					cnfChnDIn							: in		std_logic;
					cnfChnDOut							: out		std_logic;
					cnfChnShiftEn						: in		std_logic;
					cnfChnLoadEn						: in		std_logic;
					cnfChnReadBackEn					: in		std_logic);
	end component;

	component ConfigurableRegister is
		port (	clk									: in		std_logic;
					nReset								: in		std_logic;
					d										: in		std_logic;
					en										: in		std_logic;
					q										: out		std_logic;
					cnfChnDIn							: in		std_logic;
					cnfChnDOut							: out		std_logic;
					cnfChnShiftEn						: in		std_logic;
					cnfChnLoadEn						: in		std_logic;
					cnfChnReadBackEn					: in		std_logic);
	end component;
	
	component ConfigurableSRRegister is
		port (	clk									: in		std_logic;
					nReset								: in		std_logic;
					d										: in		std_logic;
					en										: in		std_logic;
					s										: in		std_logic;
					r										: in		std_logic;
					q										: out		std_logic;
					cnfChnDIn							: in		std_logic;
					cnfChnDOut							: out		std_logic;
					cnfChnShiftEn						: in		std_logic;
					cnfChnLoadEn						: in		std_logic;
					cnfChnReadBackEn					: in		std_logic);
	end component;

	component ConfigurableMemory is
		port (	clk									: in		std_logic;
					nReset								: in		std_logic;
					enable								: in		std_logic;
					wEn									: in		std_logic;
					add									: in		std_logic_vector(12 downto 0);
					dIn									: in		std_logic_vector(15 downto 0);
					dOutLU								: out		std_logic_vector(15 downto 0);
					dOutU									: out		std_logic_vector(15 downto 0);
					cnf_dIn								: in		std_logic;
					cnf_dOut								: out		std_logic;
					cnf_shiftEn							: in		std_logic;
					cnf_addDataSel						: in		std_logic;
					cnf_readEn							: in		std_logic;
					cnf_writeEn							: in		std_logic);
	end component;

	component ConfigArbiter is
		port (	clk									: in		std_logic;
					nReset								: in		std_logic;
					cnf_dIn								: in		std_logic;
					cnf_dOut								: out		std_logic;
					cnf_shiftEn							: in		std_logic;
					cnf_ctrl								: in		std_logic_vector(1 downto 0);
					cnf_target							: in		CONFIGTYPE_CAP_TARGET;
					cnf_targetSetEn					: in		std_logic;
					fpga_logicCnfChain_dIn			: in		std_logic;
					fpga_logicCnfChain_dOut			: out		std_logic;
					fpga_logicCnfChain_shiftEn		: out		std_logic;
					fpga_logicCnfChain_loadEn		: out		std_logic;
					fpga_logicCnfChain_readBackEn	: out		std_logic;
					uP_logicCnfChain_dIn				: in		std_logic;
					uP_logicCnfChain_dOut			: out		std_logic;
					uP_logicCnfChain_shiftEn		: in		std_logic;
					uP_logicCnfChain_loadEn			: in		std_logic;
					uP_logicCnfChain_readBackEn	: in		std_logic;
					uP_logicCnfChain_lockRequest	: in		std_logic;
					uP_logicCnfChain_lock			: out		std_logic;
					fpga_logicSelChain_dIn			: in		std_logic;
					fpga_logicSelChain_dOut			: out		std_logic;
					fpga_logicSelChain_shiftEn		: out		std_logic;
					uP_logicSelChain_dIn				: in		std_logic;
					uP_logicSelChain_dOut			: out		std_logic;
					uP_logicSelChain_shiftEn		: in		std_logic;
					uP_logicSelChain_feedbackEn	: in		std_logic;
					uP_logicSelChain_lockRequest	: in		std_logic;
					uP_logicSelChain_lock			: out		std_logic;
					fpga_routeCnfChain_dIn			: in		std_logic;
					fpga_routeCnfChain_dOut			: out		std_logic;
					fpga_routeCnfChain_shiftEn		: out		std_logic;
					fpga_routeCnfChain_loadEn		: out		std_logic;
					fpga_routeCnfChain_readBackEn	: out		std_logic;
					uP_routeCnfChain_dIn				: in		std_logic;
					uP_routeCnfChain_dOut			: out		std_logic;
					uP_routeCnfChain_shiftEn		: in		std_logic;
					uP_routeCnfChain_loadEn			: in		std_logic;
					uP_routeCnfChain_readBackEn	: in		std_logic;
					uP_routeCnfChain_lockRequest	: in		std_logic;
					uP_routeCnfChain_lock			: out		std_logic;
					fpga_routeSelChain_dIn			: in		std_logic;
					fpga_routeSelChain_dOut			: out		std_logic;
					fpga_routeSelChain_shiftEn		: out		std_logic;
					uP_routeSelChain_dIn				: in		std_logic;
					uP_routeSelChain_dOut			: out		std_logic;
					uP_routeSelChain_shiftEn		: in		std_logic;
					uP_routeSelChain_feedbackEn	: in		std_logic;
					uP_routeSelChain_lockRequest	: in		std_logic;
					uP_routeSelChain_lock			: out		std_logic;
					uP_memCnf_dIn						: in		std_logic;
					uP_memCnf_dOut						: out		std_logic;
					uP_memCnf_shiftEn					: out		std_logic;
					uP_memCnf_addDataSel				: out		std_logic;
					uP_memCnf_readEn					: out		std_logic;
					uP_memCnf_writeEn					: out		std_logic;
					uP_control_nReset					: out		std_logic;
					uP_control_enable					: out		std_logic);
		end component;

end ConfigPkg;

package body ConfigPkg is

---------------------------------------------------------------------------------------------------------------
-- Conversion: std_logic <-> Config Chain
---------------------------------------------------------------------------------------------------------------
-- {{{
function to_CONFIGTYPE_CHAIN( value : std_logic ) return CONFIGTYPE_CHAIN is
	variable result : CONFIGTYPE_CHAIN;
begin

	case value is
		when '1'		=> result := CONFIG_CNF_CHAIN;
		when '0'		=> result := CONFIG_SEL_CHAIN;
		when others	=>
			assert false
			report "to_CONFIGTYPE_CHAIN: unconvertable std_logic value"
			severity failure;
	end case;

	return result;

end function to_CONFIGTYPE_CHAIN;


function to_std_logic( value : CONFIGTYPE_CHAIN ) return std_logic is
	variable result : std_logic;
begin

	case value is
		when CONFIG_CNF_CHAIN	=> result := '1';
		when CONFIG_SEL_CHAIN	=> result := '0';
	end case;

	return result;

end function to_std_logic;
-- }}}

---------------------------------------------------------------------------------------------------------------
-- Conversion: std_logic <-> Config Target
---------------------------------------------------------------------------------------------------------------
-- {{{
function to_CONFIGTYPE_TARGET( value : std_logic ) return CONFIGTYPE_TARGET is
	variable result : CONFIGTYPE_TARGET;
begin

	case value is
		when '1'		=> result := CONFIG_LOGIC_TARGET;
		when '0'		=> result := CONFIG_ROUTE_TARGET;
		when others	=>
			assert false
			report "to_CONFIGTYPE_TARGET: unconvertable std_logic value"
			severity failure;
	end case;

	return result;

end function to_CONFIGTYPE_TARGET;


function to_std_logic( value : CONFIGTYPE_TARGET ) return std_logic is
	variable result : std_logic;
begin

	case value is
		when CONFIG_LOGIC_TARGET	=> result := '1';
		when CONFIG_ROUTE_TARGET	=> result := '0';
	end case;

	return result;

end function to_std_logic;
-- }}}

---------------------------------------------------------------------------------------------------------------
-- Conversion: std_logic_vector <-> Config CAP Target
---------------------------------------------------------------------------------------------------------------
function to_CONFIGTYPE_CAP_TARGET( value : std_logic_vector(2 downto 0) ) return CONFIGTYPE_CAP_TARGET is
	variable result : CONFIGTYPE_CAP_TARGET;
begin

	case value is
		when b"000" => result := CONFIG_CAP_TARGET_NULL;
		when b"001" => result := CONFIG_CAP_TARGET_UP_CONTROL;
		when b"010" => result := CONFIG_CAP_TARGET_LOGIC_CNF_CHAIN;
		when b"011" => result := CONFIG_CAP_TARGET_LOGIC_SEL_CHAIN;
		when b"100" => result := CONFIG_CAP_TARGET_ROUTE_CNF_CHAIN;
		when b"101" => result := CONFIG_CAP_TARGET_ROUTE_SEL_CHAIN;
		when b"110" => result := CONFIG_CAP_TARGET_UP_MEM_DATA;
		when b"111" => result := CONFIG_CAP_TARGET_UP_MEM_ADDRESS;
		when others	=>
			assert false
			report "to_CONFIGTYPE_CAP_TARGET: unconvertable std_logic value"
			severity failure;
	end case;

	return result;

end function to_CONFIGTYPE_CAP_TARGET;

function to_std_logic_vector( value : CONFIGTYPE_CAP_TARGET ) return std_logic_vector is
	variable result : std_logic_vector(2 downto 0);
begin

	case value is
		when CONFIG_CAP_TARGET_NULL				=> result := b"000";
		when CONFIG_CAP_TARGET_UP_CONTROL		=> result := b"001";
		when CONFIG_CAP_TARGET_LOGIC_CNF_CHAIN	=> result := b"010";
		when CONFIG_CAP_TARGET_LOGIC_SEL_CHAIN	=> result := b"011";
		when CONFIG_CAP_TARGET_ROUTE_CNF_CHAIN	=> result := b"100";
		when CONFIG_CAP_TARGET_ROUTE_SEL_CHAIN	=> result := b"101";
		when CONFIG_CAP_TARGET_UP_MEM_DATA		=> result := b"110";
		when CONFIG_CAP_TARGET_UP_MEM_ADDRESS	=> result := b"111";
	end case;

	return result;

end function to_std_logic_vector;

end ConfigPkg;

