---------------------------------------------------------------------------------------------------------------
-- RISA Project
-- Author: A.Greensted
-- Module:
-- Description:
-- History:
---------------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ConfigSelectUnit is
	port (	clk					: in		std_logic;
				nReset				: in		std_logic;

				selChnDIn				: in		std_logic;
				selChnDOut				: out		std_logic;
				selChnShiftEn			: in		std_logic;
				
				-- Config Chain
				cnfChnDIn				: in		std_logic;
				cnfChnDOut				: out		std_logic;
				cnfChnShiftEn			: in		std_logic;
				cnfChnLoadEn			: in		std_logic;
				cnfChnReadBackEn		: in		std_logic;

				cnfChnDInBP				: in		std_logic;
				cnfChnDOutBP			: out		std_logic;
				cnfChnShiftEnBP		: out		std_logic;
				cnfChnLoadEnBP			: out		std_logic;
				cnfChnReadBackEnBP	: out		std_logic);

end ConfigSelectUnit;

architecture General of ConfigSelectUnit is

	signal selectBit	: std_logic;
	signal cnfChain	: std_logic;

begin

SelChainReg : process(clk)
begin

	if (clk'event and clk='1') then

		if (nReset='0') then
			selectBit <= '0';
		
		elsif (selChnShiftEn='1') then
			selectBit <= selChnDIn;

		end if;
	end if;

end process;

selChnDOut <= selectBit;


CnfChainReg : process(clk)
begin

	if (clk'event and clk='1') then

		if (nReset='0') then
			cnfChain <= '0';
		
		elsif (cnfChnShiftEn='1') then
			cnfChain <= cnfChnDIn;

		end if;

	end if;

end process;

cnfChnDOutBP <= cnfChain;


Selector : process(selectBit, cnfChain, cnfChnDInBP, cnfChnShiftEn, cnfChnLoadEn, cnfChnReadBackEn)
begin
	if (selectBit='1') then
		cnfChnLoadEnBP			<= cnfChnLoadEn;
		cnfChnReadBackEnBP	<= cnfChnReadBackEn;
		cnfChnShiftEnBP		<= cnfChnShiftEn;

		cnfChnDOut				<= cnfChnDInBP;

	else
		cnfChnLoadEnBP			<= '0';
		cnfChnReadBackEnBP	<= '0';
		cnfChnShiftEnBP		<= '0';

		cnfChnDOut				<= cnfChain;

	end if;

end process;

end general;
