---------------------------------------------------------------------------------------------------------------
-- RISA Project
-- Author: A.Greensted
-- Module: ConfigurableRegister
-- Description: A configurable 1 bit register
-- History:
---------------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ConfigurableRegister is
	port (	clk					: in		std_logic;
				nReset				: in		std_logic;
                              
				d						: in		std_logic;
				en						: in		std_logic;
				q						: out		std_logic;

				-- Config Chain
				cnfChnDIn			: in		std_logic;
				cnfChnDOut			: out		std_logic;
				cnfChnShiftEn		: in		std_logic;
				cnfChnLoadEn		: in		std_logic;
				cnfChnReadBackEn	: in		std_logic);

end ConfigurableRegister;

architecture General of ConfigurableRegister is

	signal cnfChnData			: std_logic;
	signal regOut				: std_logic;

begin

ChainReg : process(clk)
begin

	if (clk'event and clk='1') then

		if (nReset='0') then
			cnfChnData <= '0';
		
		elsif (cnfChnReadBackEn='1') then
			cnfChnData <= regOut;

		elsif (cnfChnShiftEn='1') then
			cnfChnData <= cnfChnDIn;

		end if;

	end if;

end process;


ConfReg : process(clk)
begin

	if (clk'event and clk='1') then

		if (nReset='0') then
			regOut <= '0';

		elsif (cnfChnLoadEn='1') then
			regOut <= cnfChnData;

		elsif (en='1') then
			regOut <= d;

		end if;

	end if;

end process;

cnfChnDOut <= cnfChnData;
q <= regOut;

end General;
