---------------------------------------------------------------------------------------------------------------
-- RISA Project
-- Author: A.Greensted
-- Module: RISAPkg
-- Description: RISA Package
-- History:
---------------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library ConfigLib;
use ConfigLib.ConfigPkg.all;

package RISAPkg is

	-----------------------------------------------------------------------------------------
	-- Components
	-----------------------------------------------------------------------------------------
	component RISA is
		port (	clk					: in		std_logic;
					nReset				: in		std_logic;
					-- Config Access Port
					cap_clk				: in		std_logic;
					cap_nReset			: in		std_logic;
					cap_enable			: in		std_logic;
					cap_ctrlDataSel	: in		std_logic;
					cap_shiftExecSel	: in		std_logic;
					cap_dIn				: in		std_logic;
					cap_dOut				: out		std_logic;
					cap_busy				: out		std_logic;
					-- FPGA: IO Connections
					fIO_N_in				: in		std_logic_vector(11 downto 0);
					fIO_N_out			: out		std_logic_vector(11 downto 0);
					fIO_N_outEn			: out		std_logic_vector(11 downto 0);
					fIO_S_in				: in		std_logic_vector(11 downto 0);
					fIO_S_out			: out		std_logic_vector(11 downto 0);
					fIO_S_outEn			: out		std_logic_vector(11 downto 0);
					fIO_E_in				: in		std_logic_vector(11 downto 0);
					fIO_E_out			: out		std_logic_vector(11 downto 0);
					fIO_E_outEn			: out		std_logic_vector(11 downto 0);
					fIO_W_in				: in		std_logic_vector(11 downto 0);
					fIO_W_out			: out		std_logic_vector(11 downto 0);
					fIO_W_outEn			: out		std_logic_vector(11 downto 0);
					-- FPGA: Gloabl IO Connections
					fGIn					: in		std_logic_vector(1 downto 0);
					-- Snap: Snap Links
					sLnk_N_txOut		: out		std_logic;
					sLnk_N_rxIn			: in		std_logic;
					sLnk_N_CTSOut		: out		std_logic;
					sLnk_N_CTSIn		: in		std_logic;
					sLnk_E_txOut		: out		std_logic;
					sLnk_E_rxIn			: in		std_logic;
					sLnk_E_CTSOut		: out		std_logic;
					sLnk_E_CTSIn		: in		std_logic;
					sLnk_S_txOut		: out		std_logic;
					sLnk_S_rxIn			: in		std_logic;
					sLnk_S_CTSOut		: out		std_logic;
					sLnk_S_CTSIn		: in		std_logic;
					sLnk_W_txOut		: out		std_logic;
					sLnk_W_rxIn			: in		std_logic;
					sLnk_W_CTSOut		: out		std_logic;
					sLnk_W_CTSIn		: in		std_logic;
					-- Snap: System UART
					sysUART_txOut		: out		std_logic;
					sysUART_rxIn		: in		std_logic;

					-- Snap: IO Ports
					sIO_N_in				: in		std_logic_vector(7 downto 0);
					sIO_N_out			: out		std_logic_vector(7 downto 0);
					sIO_N_outEn			: out		std_logic_vector(7 downto 0);
					sIO_E_in				: in		std_logic_vector(7 downto 0);
					sIO_E_out			: out		std_logic_vector(7 downto 0);
					sIO_E_outEn			: out		std_logic_vector(7 downto 0);
					sIO_W_in				: in		std_logic_vector(7 downto 0);
					sIO_W_out			: out		std_logic_vector(7 downto 0);
					sIO_W_outEn			: out		std_logic_vector(7 downto 0);
					-- Snap: Counters
					cnt_0_extCountEn	: in		std_logic;
					cnt_0_match0Out	: out		std_logic;
					cnt_0_match1Out	: out		std_logic;
					cnt_1_extCountEn	: in		std_logic;
					cnt_1_match0Out	: out		std_logic;
					cnt_1_match1Out	: out		std_logic;
					-- Snap: Flags & IRQs
					extFlagsIn			: in		std_logic_vector(1 downto 0);
					extIRQsIn			: in		std_logic_vector(1 downto 0));
	end component;

	component RISA_Wrap is
		port (	clk					: in		std_logic;
					nReset				: in		std_logic;
					-- Config Access Port
					cap_clk				: in		std_logic;
					cap_nReset			: in		std_logic;
					cap_enable			: in		std_logic;
					cap_ctrlDataSel	: in		std_logic;
					cap_shiftExecSel	: in		std_logic;
					cap_dIn				: in		std_logic;
					cap_dOut				: out		std_logic;
					cap_busy				: out		std_logic;
					-- FPGA: IO Connections
					fIO_N					: inout	std_logic_vector(11 downto 0);
					fIO_S					: inout	std_logic_vector(11 downto 0);
					fIO_E					: inout	std_logic_vector(11 downto 0);
					fIO_W					: inout	std_logic_vector(11 downto 0);
					-- FPGA: Gloabl IO Connections
					fGIn					: in		std_logic_vector(1 downto 0);
					-- Snap: Snap Links
					sLnk_N_txOut		: out		std_logic;
					sLnk_N_rxIn			: in		std_logic;
					sLnk_N_CTSOut		: out		std_logic;
					sLnk_N_CTSIn		: in		std_logic;
					sLnk_E_txOut		: out		std_logic;
					sLnk_E_rxIn			: in		std_logic;
					sLnk_E_CTSOut		: out		std_logic;
					sLnk_E_CTSIn		: in		std_logic;
					sLnk_S_txOut		: out		std_logic;
					sLnk_S_rxIn			: in		std_logic;
					sLnk_S_CTSOut		: out		std_logic;
					sLnk_S_CTSIn		: in		std_logic;
					sLnk_W_txOut		: out		std_logic;
					sLnk_W_rxIn			: in		std_logic;
					sLnk_W_CTSOut		: out		std_logic;
					sLnk_W_CTSIn		: in		std_logic;
					-- Snap: System UART
					sysUART_txOut		: out		std_logic;
					sysUART_rxIn		: in		std_logic;
					-- Snap: IO Ports
					sIO_N					: inout	std_logic_vector(7 downto 0);
					sIO_E					: inout	std_logic_vector(7 downto 0);
					sIO_W					: inout	std_logic_vector(7 downto 0);
					-- Snap: Counters
					cnt_0_extCountEn	: in		std_logic;
					cnt_0_match0Out	: out		std_logic;
					cnt_0_match1Out	: out		std_logic;
					cnt_1_extCountEn	: in		std_logic;
					cnt_1_match0Out	: out		std_logic;
					cnt_1_match1Out	: out		std_logic;
					-- Snap: Flags & IRQs
					extFlagsIn			: in		std_logic_vector(1 downto 0);
					extIRQsIn			: in		std_logic_vector(1 downto 0));
	end component;

end RISAPkg;

package body RISAPkg is

end RISAPkg;
