---------------------------------------------------------------------------------------------------------------
-- RISA Project
-- Author: A.Greensted
-- Module:
-- Description:
-- History:
---------------------------------------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library SnapLib;
use SnapLib.SnapPkg.all;

entity RegisterFile is
	port (	clk								: in		std_logic;
				enable							: in		std_logic;
				nReset							: in		std_logic;

				portAWEn							: in		std_logic;
				portBREn							: in		std_logic;

				portAAdd							: in		std_logic_vector(6 downto 0);
				portBAdd							: in		std_logic_vector(6 downto 0);

				portADIn							: in		std_logic_vector(15 downto 0);

				portADOut						: out		std_logic_vector(15 downto 0);
				portBDOut						: out		std_logic_vector(15 downto 0);

				-- Program Counter
				pc_pcOut							: out		SNAPTYPE_MEM_ADD;
				pc_isrBranchEn					: in		std_logic;
				pc_activePC						: in		SNAPTYPE_MEM_ADD;
				pc_braPCWEn						: in		std_logic;
				pc_isrPCWEn						: in		std_logic;

				-- Flags Registers
				fReg_extFlagsIn				: in		SNAPTYPE_EXTERNAL_FLAGS;
				fReg_aluFlagsWEn				: in		std_logic;
				fReg_aluFlagsIn				: in		SNAPTYPE_ALU_FLAGS;
				fReg_conditionCodeWEn		: in		std_logic;
				fReg_conditionCodeIn			: in		SNAPTYPE_CONDITION_CODE;
				fReg_intFlagsWEn				: in		std_logic;
				fReg_intFlagsIn				: in		std_logic_vector(7 downto 0);
				fReg_intFlagsMaskIn			: in		std_logic_vector(7 downto 0);
				fReg_setGIEEn					: in		std_logic;
				fReg_clrGIEEn					: in		std_logic;
				fReg_flagsOut					: out		SNAPTYPE_FLAGS;
				fReg_conditionCodeOut		: out		SNAPTYPE_CONDITION_CODE;

				-- Interrupt Controller
				intC_extIRQsIn					: in		std_logic_vector(7 downto 0);
				intC_interruptRequest		: out		std_logic;

				-- Snap Links
				sLnk_N_linkTxOut				: out		std_logic;
				sLnk_N_linkRxIn				: in		std_logic;
				sLnk_N_linkCTSOut				: out		std_logic;
				sLnk_N_linkCTSIn				: in		std_logic;
				sLnk_E_linkTxOut				: out		std_logic;
				sLnk_E_linkRxIn				: in		std_logic;
				sLnk_E_linkCTSOut				: out		std_logic;
				sLnk_E_linkCTSIn				: in		std_logic;
				sLnk_S_linkTxOut				: out		std_logic;
				sLnk_S_linkRxIn				: in		std_logic;
				sLnk_S_linkCTSOut				: out		std_logic;
				sLnk_S_linkCTSIn				: in		std_logic;
				sLnk_W_linkTxOut				: out		std_logic;
				sLnk_W_linkRxIn				: in		std_logic;
				sLnk_W_linkCTSOut				: out		std_logic;
				sLnk_W_linkCTSIn				: in		std_logic;

				-- Counter 0
				cnt_0_extCountEn				: in		std_logic;
				cnt_0_match0Out				: out		std_logic;
				cnt_0_match1Out				: out		std_logic;

				-- Counter 1
				cnt_1_extCountEn				: in		std_logic;
				cnt_1_match0Out				: out		std_logic;
				cnt_1_match1Out				: out		std_logic;

				-- System UART
				sUART_serialTxOut				: out		std_logic;
				sUART_serialRxIn				: in		std_logic;

				-- Randon Number Generator
				rng_randomDataIn				: in		std_logic_vector(15 downto 0);

				-- IO Ports
				ioP_N_dIn						: in		std_logic_vector(7 downto 0);
				ioP_N_outEn						: out		std_logic_vector(7 downto 0);
				ioP_N_dOut						: out		std_logic_vector(7 downto 0);
				ioP_E_dIn						: in		std_logic_vector(7 downto 0);
				ioP_E_outEn						: out		std_logic_vector(7 downto 0);
				ioP_E_dOut						: out		std_logic_vector(7 downto 0);
				ioP_W_dIn						: in		std_logic_vector(7 downto 0);
				ioP_W_outEn						: out		std_logic_vector(7 downto 0);
				ioP_W_dOut						: out		std_logic_vector(7 downto 0);

				-- Fabric Ports
				fabIO_N_dIn						: in		std_logic_vector(5 downto 0);
				fabIO_N_dOut					: out		std_logic_vector(5 downto 0);
				fabIO_S_dIn						: in		std_logic_vector(5 downto 0);
				fabIO_S_dOut					: out		std_logic_vector(5 downto 0);
				fabIO_E_dIn						: in		std_logic_vector(5 downto 0);
				fabIO_E_dOut					: out		std_logic_vector(5 downto 0);
				fabIO_W_dIn						: in		std_logic_vector(5 downto 0);
				fabIO_W_dOut					: out		std_logic_vector(5 downto 0);

				-- Fabric CAP Logic
				cap_L_cnfChainLockRequest	: out		std_logic;
				cap_L_cnfChainLock			: in 		std_logic;
				cap_L_cnfChainShiftEn		: out		std_logic;
				cap_L_cnfChainLoadEn			: out		std_logic;
				cap_L_cnfChainReadBackEn	: out		std_logic;

				cap_L_cnfChainOut				: out		std_logic;
				cap_L_cnfChainIn				: in		std_logic;

				cap_L_selChainLockRequest	: out 	std_logic;
				cap_L_selChainLock			: in 		std_logic;
				cap_L_selChainShiftEn		: out		std_logic;
				cap_L_selChainFeedbackEn	: out		std_logic;

				cap_L_selChainOut				: out		std_logic;
				cap_L_selChainIn				: in		std_logic;

				-- Fabric CAP Route
				cap_R_cnfChainLockRequest	: out		std_logic;
				cap_R_cnfChainLock			: in 		std_logic;
				cap_R_cnfChainShiftEn		: out		std_logic;
				cap_R_cnfChainLoadEn			: out		std_logic;
				cap_R_cnfChainReadBackEn	: out		std_logic;

				cap_R_cnfChainOut				: out		std_logic;
				cap_R_cnfChainIn				: in		std_logic;

				cap_R_selChainLockRequest	: out 	std_logic;
				cap_R_selChainLock			: in 		std_logic;
				cap_R_selChainShiftEn		: out		std_logic;
				cap_R_selChainFeedbackEn	: out		std_logic;

				cap_R_selChainOut				: out		std_logic;
				cap_R_selChainIn				: in		std_logic);

end RegisterFile;

architecture General of RegisterFile is

	-- Working Register
	signal wReg_portAWEn				: std_logic;
	signal wReg_portADOut			: std_logic_vector(15 downto 0);
	signal wReg_portBDOut			: std_logic_vector(15 downto 0);

	-- Program Counter
	signal pc_pcWEn					: std_logic;
	signal pc_pcOut_sig				: SNAPTYPE_MEM_ADD;
	signal pc_braPCOut				: SNAPTYPE_MEM_ADD;
	signal pc_isrPCOut				: SNAPTYPE_MEM_ADD;

	-- Flag Register
	signal fReg_flagReg0WEn			: std_logic;
	signal fReg_flagReg0Out			: std_logic_vector(15 downto 0);
	signal fReg_flagReg1WEn			: std_logic;
	signal fReg_flagReg1Out			: std_logic_vector(15 downto 0);

	-- Interrupt Controller
	signal intC_flagsReg0WEn		: std_logic;
	signal intC_flagsReg0Out		: std_logic_vector(15 downto 0);
	signal intC_flagsReg1WEn		: std_logic;
	signal intC_flagsReg1Out		: std_logic_vector(15 downto 0);
	signal intC_enableReg0WEn		: std_logic;
	signal intC_enableReg0Out		: std_logic_vector(15 downto 0);
	signal intC_enableReg1WEn		: std_logic;
	signal intC_enableReg1Out		: std_logic_vector(15 downto 0);
	signal intC_intClr0WEn			: std_logic;
	signal intC_intClr1WEn			: std_logic;
	signal intC_intSet0WEn			: std_logic;
	signal intC_intSet1WEn			: std_logic;
	signal intC_interruptsIn		: std_logic_vector(31 downto 0);

	-- Snap Link North
	signal sLnk_N_controlRegWEn	: std_logic;
	signal sLnk_N_controlRegOut	: std_logic_vector(15 downto 0);
	signal sLnk_N_dataRegWEn		: std_logic;
	signal sLnk_N_dataRegREn		: std_logic;
	signal sLnk_N_dataRegOut		: std_logic_vector(15 downto 0);
	signal sLnk_N_newDataIF			: std_logic;
	signal sLnk_N_rxFifoFullIF		: std_logic;
	signal sLnk_N_txFreeIF			: std_logic;

	-- Snap Link East
	signal sLnk_E_controlRegWEn	: std_logic;
	signal sLnk_E_controlRegOut	: std_logic_vector(15 downto 0);
	signal sLnk_E_dataRegWEn		: std_logic;
	signal sLnk_E_dataRegREn		: std_logic;
	signal sLnk_E_dataRegOut		: std_logic_vector(15 downto 0);
	signal sLnk_E_newDataIF			: std_logic;
	signal sLnk_E_rxFifoFullIF		: std_logic;
	signal sLnk_E_txFreeIF			: std_logic;

	-- Snap Link South
	signal sLnk_S_controlRegWEn	: std_logic;
	signal sLnk_S_controlRegOut	: std_logic_vector(15 downto 0);
	signal sLnk_S_dataRegWEn		: std_logic;
	signal sLnk_S_dataRegREn		: std_logic;
	signal sLnk_S_dataRegOut		: std_logic_vector(15 downto 0);
	signal sLnk_S_newDataIF			: std_logic;
	signal sLnk_S_rxFifoFullIF		: std_logic;
	signal sLnk_S_txFreeIF			: std_logic;

	-- Snap Link West
	signal sLnk_W_controlRegWEn	: std_logic;
	signal sLnk_W_controlRegOut	: std_logic_vector(15 downto 0);
	signal sLnk_W_dataRegWEn		: std_logic;
	signal sLnk_W_dataRegREn		: std_logic;
	signal sLnk_W_dataRegOut		: std_logic_vector(15 downto 0);
	signal sLnk_W_newDataIF			: std_logic;
	signal sLnk_W_rxFifoFullIF		: std_logic;
	signal sLnk_W_txFreeIF			: std_logic;

	-- Counter 0
	signal cnt_0_controlRegWEn		: std_logic;
	signal cnt_0_controlRegOut		: std_logic_vector(15 downto 0);
	signal cnt_0_counterWEn			: std_logic;
	signal cnt_0_counterOut			: std_logic_vector(15 downto 0);
	signal cnt_0_compareReg0WEn	: std_logic;
	signal cnt_0_compareReg0Out	: std_logic_vector(15 downto 0);
	signal cnt_0_compareReg1WEn	: std_logic;
	signal cnt_0_compareReg1Out	: std_logic_vector(15 downto 0);
	signal cnt_0_overflowIF			: std_logic;
	signal cnt_0_match0IF			: std_logic;
	signal cnt_0_match1IF			: std_logic;

	-- Counter 1
	signal cnt_1_controlRegWEn		: std_logic;
	signal cnt_1_controlRegOut		: std_logic_vector(15 downto 0);
	signal cnt_1_counterWEn			: std_logic;
	signal cnt_1_counterOut			: std_logic_vector(15 downto 0);
	signal cnt_1_compareReg0WEn	: std_logic;
	signal cnt_1_compareReg0Out	: std_logic_vector(15 downto 0);
	signal cnt_1_compareReg1WEn	: std_logic;
	signal cnt_1_compareReg1Out	: std_logic_vector(15 downto 0);
	signal cnt_1_overflowIF			: std_logic;
	signal cnt_1_match0IF			: std_logic;
	signal cnt_1_match1IF			: std_logic;

	-- Watch Dog Timer
	signal wdt_controlRegWEn		: std_logic;
	signal wdt_controlRegOut		: std_logic_vector(15 downto 0);
	signal wdt_timerWEn				: std_logic;
	signal wdt_timerOut				: std_logic_vector(15 downto 0);
	signal wdt_timeoutIF				: std_logic;

	-- System UART
	signal sUART_controlRegWEn		: std_logic;
	signal sUART_controlRegOut		: std_logic_vector(15 downto 0);
	signal sUART_dataRegWEn			: std_logic;
	signal sUART_dataRegREn			: std_logic;
	signal sUART_dataRegOut			: std_logic_vector(15 downto 0);
	signal sUART_baudRegWEn			: std_logic;
	signal sUART_baudRegOut			: std_logic_vector(15 downto 0);
	signal sUART_newDataIF			: std_logic;
	signal sUART_rxFifoFullIF		: std_logic;
	signal sUART_txFreeIF			: std_logic;

	-- IO Ports
	signal ioP_N_dataRegWEn			: std_logic;
	signal ioP_N_dataRegOut			: std_logic_vector(7 downto 0);
	signal ioP_N_dataOutEnWEn		: std_logic;
	signal ioP_N_dataOutEnOut		: std_logic_vector(7 downto 0);
	signal ioP_E_dataRegWEn			: std_logic;
	signal ioP_E_dataRegOut			: std_logic_vector(7 downto 0);
	signal ioP_E_dataOutEnWEn		: std_logic;
	signal ioP_E_dataOutEnOut		: std_logic_vector(7 downto 0);
	signal ioP_W_dataRegWEn			: std_logic;
	signal ioP_W_dataRegOut			: std_logic_vector(7 downto 0);
	signal ioP_W_dataOutEnWEn		: std_logic;
	signal ioP_W_dataOutEnOut		: std_logic_vector(7 downto 0);

	-- Fabric IO
	signal fabIO_N_dataRegWEn		: std_logic;
	signal fabIO_N_dataRegOut		: std_logic_vector(5 downto 0);
	signal fabIO_S_dataRegWEn		: std_logic;
	signal fabIO_S_dataRegOut		: std_logic_vector(5 downto 0);
	signal fabIO_E_dataRegWEn		: std_logic;
	signal fabIO_E_dataRegOut		: std_logic_vector(5 downto 0);
	signal fabIO_W_dataRegWEn		: std_logic;
	signal fabIO_W_dataRegOut		: std_logic_vector(5 downto 0);

	-- Fabric CAP Logic
	signal cap_L_controlRegWEn		: std_logic;
	signal cap_L_controlRegOut		: std_logic_vector(15 downto 0);
	signal cap_L_dataRegWEn			: std_logic;
	signal cap_L_dataRegOut			: std_logic_vector(15 downto 0);

	-- Fabric CAP Route
	signal cap_R_controlRegWEn		: std_logic;
	signal cap_R_controlRegOut		: std_logic_vector(15 downto 0);
	signal cap_R_dataRegWEn			: std_logic;
	signal cap_R_dataRegOut			: std_logic_vector(15 downto 0);

begin

---------------------------------------------------------------------------------------------------------------
-- Component Instatiation
---------------------------------------------------------------------------------------------------------------
WREG : WorkingRegisters
port map (	clk						=> clk,
				enable					=> enable,
				portADIn					=> portADIn,
				portAWEn					=> wReg_portAWEn,
				portAAdd					=> portAAdd(4 downto 0),
				portBAdd					=> portBAdd(4 downto 0),
				portADOut				=> wReg_portADOut,
				portBDOut				=> wReg_portBDOut);

PC : ProgramCounter
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(SNAPTYPE_MEM_ADD'range),
				isrBranchEn				=> pc_isrBranchEn,
				pcWEn						=> pc_pcWEn,
				pcOut						=> pc_pcOut_sig,
				activePC					=> pc_activePC,
				braPCWEn					=> pc_braPCWEn,
				braPCOut					=> pc_braPCOut,
				isrPCWEn					=> pc_isrPCWEn,
				isrPCOut					=> pc_isrPCOut);

pc_pcOut <= pc_pcOut_sig;

FREG : FlagsRegisters
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(12 downto 0),
				flagReg0WEn				=> fReg_flagReg0WEn,
				flagReg0Out				=> fReg_flagReg0Out,
				flagReg1WEn				=> fReg_flagReg1WEn,
				flagReg1Out				=> fReg_flagReg1Out,
				extFlagsIn				=> fReg_extFlagsIn,
				aluFlagsWEn				=> fReg_aluFlagsWEn,
				aluFlagsIn				=> fReg_aluFlagsIn,
				conditionCodeWEn		=> fReg_conditionCodeWEn,
				conditionCodeIn		=> fReg_conditionCodeIn,
				intFlagsWEn				=> fReg_intFlagsWEn,
				intFlagsIn				=> fReg_intFlagsIn,
				intFlagsMaskIn			=> fReg_intFlagsMaskIn,
				setGIEEn					=> fReg_setGIEEn,
				clrGIEEn					=> fReg_clrGIEEn,
				flagsOut					=> fReg_flagsOut,
				conditionCodeOut		=> fReg_conditionCodeOut);

INTC : InterruptController
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				flagsReg0WEn			=> intC_flagsReg0WEn,
				flagsReg0Out			=> intC_flagsReg0Out,
				flagsReg1WEn			=> intC_flagsReg1WEn,
				flagsReg1Out			=> intC_flagsReg1Out,
				enableReg0WEn			=> intC_enableReg0WEn,
				enableReg0Out			=> intC_enableReg0Out,
				enableReg1WEn			=> intC_enableReg1WEn,
				enableReg1Out			=> intC_enableReg1Out,
				intClr0WEn				=> intC_intClr0WEn,
				intClr1WEn				=> intC_intClr1WEn,
				intSet0WEn				=> intC_intSet0WEn,
				intSet1WEn				=> intC_intSet1WEn,
				interruptsIn			=> intC_interruptsIn,
				interruptRequest		=> intC_interruptRequest);

SLNK_N : SnapLink
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> sLnk_N_controlRegWEn,
				controlRegOut			=> sLnk_N_controlRegOut,
				dataRegWEn				=> sLnk_N_dataRegWEn,
				dataRegREn				=> sLnk_N_dataRegREn,
				dataRegOut				=> sLnk_N_dataRegOut,
				newDataIF				=> sLnk_N_newDataIF,
				rxFifoFullIF			=> sLnk_N_rxFifoFullIF,
				txFreeIF					=> sLnk_N_txFreeIF,
				linkTxOut				=> sLnk_N_linkTxOut,
				linkRxIn					=> sLnk_N_linkRxIn,
				linkCTSOut				=> sLnk_N_linkCTSOut,
				linkCTSIn				=> sLnk_N_linkCTSIn);

SLNK_E : SnapLink
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> sLnk_E_controlRegWEn,
				controlRegOut			=> sLnk_E_controlRegOut,
				dataRegWEn				=> sLnk_E_dataRegWEn,
				dataRegREn				=> sLnk_E_dataRegREn,
				dataRegOut				=> sLnk_E_dataRegOut,
				newDataIF				=> sLnk_E_newDataIF,
				rxFifoFullIF			=> sLnk_E_rxFifoFullIF,
				txFreeIF					=> sLnk_E_txFreeIF,
				linkTxOut				=> sLnk_E_linkTxOut,
				linkRxIn					=> sLnk_E_linkRxIn,
				linkCTSOut				=> sLnk_E_linkCTSOut,
				linkCTSIn				=> sLnk_E_linkCTSIn);

SLNK_S : SnapLink
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> sLnk_S_controlRegWEn,
				controlRegOut			=> sLnk_S_controlRegOut,
				dataRegWEn				=> sLnk_S_dataRegWEn,
				dataRegREn				=> sLnk_S_dataRegREn,
				dataRegOut				=> sLnk_S_dataRegOut,
				newDataIF				=> sLnk_S_newDataIF,
				rxFifoFullIF			=> sLnk_S_rxFifoFullIF,
				txFreeIF					=> sLnk_S_txFreeIF,
				linkTxOut				=> sLnk_S_linkTxOut,
				linkRxIn					=> sLnk_S_linkRxIn,
				linkCTSOut				=> sLnk_S_linkCTSOut,
				linkCTSIn				=> sLnk_S_linkCTSIn);

SLNK_W : SnapLink
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> sLnk_W_controlRegWEn,
				controlRegOut			=> sLnk_W_controlRegOut,
				dataRegWEn				=> sLnk_W_dataRegWEn,
				dataRegREn				=> sLnk_W_dataRegREn,
				dataRegOut				=> sLnk_W_dataRegOut,
				newDataIF				=> sLnk_W_newDataIF,
				rxFifoFullIF			=> sLnk_W_rxFifoFullIF,
				txFreeIF					=> sLnk_W_txFreeIF,
				linkTxOut				=> sLnk_W_linkTxOut,
				linkRxIn					=> sLnk_W_linkRxIn,
				linkCTSOut				=> sLnk_W_linkCTSOut,
				linkCTSIn				=> sLnk_W_linkCTSIn);

CNT0 : Counter
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> cnt_0_controlRegWEn,
				controlRegOut			=> cnt_0_controlRegOut,
				counterWEn				=> cnt_0_counterWEn,
				counterOut				=> cnt_0_counterOut,
				compareReg0WEn			=> cnt_0_compareReg0WEn,
				compareReg0Out			=> cnt_0_compareReg0Out,
				compareReg1WEn			=> cnt_0_compareReg1WEn,
				compareReg1Out			=> cnt_0_compareReg1Out,
				extCountEn				=> cnt_0_extCountEn,
				overflowIF				=> cnt_0_overflowIF,
				match0IF					=> cnt_0_match0IF,
				match1IF					=> cnt_0_match1IF,
				match0Out				=> cnt_0_match0Out,
				match1Out				=> cnt_0_match1Out);

CNT1 : Counter
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> cnt_1_controlRegWEn,
				controlRegOut			=> cnt_1_controlRegOut,
				counterWEn				=> cnt_1_counterWEn,
				counterOut				=> cnt_1_counterOut,
				compareReg0WEn			=> cnt_1_compareReg0WEn,
				compareReg0Out			=> cnt_1_compareReg0Out,
				compareReg1WEn			=> cnt_1_compareReg1WEn,
				compareReg1Out			=> cnt_1_compareReg1Out,
				extCountEn				=> cnt_1_extCountEn,
				overflowIF				=> cnt_1_overflowIF,
				match0IF					=> cnt_1_match0IF,
				match1IF					=> cnt_1_match1IF,
				match0Out				=> cnt_1_match0Out,
				match1Out				=> cnt_1_match1Out);

WDT : WatchDogTimer
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> wdt_controlRegWEn,
				controlRegOut			=> wdt_controlRegOut,
				timerWEn					=> wdt_timerWEn,
				timerOut					=> wdt_timerOut,
				timeoutIF				=> wdt_timeoutIF);

SUART : SystemUART
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> sUART_controlRegWEn,
				controlRegOut			=> sUART_controlRegOut,
				dataRegWEn				=> sUART_dataRegWEn,
				dataRegREn				=> sUART_dataRegREn,
				dataRegOut				=> sUART_dataRegOut,
				baudRegWEn				=> sUART_baudRegWEn,
				baudRegOut				=> sUART_baudRegOut,
				newDataIF				=> sUART_newDataIF,
				rxFifoFullIF			=> sUART_rxFifoFullIF,
				txFreeIF					=> sUART_txFreeIF,
				serialTxOut				=> sUART_serialTxOut,
				serialRxIn				=> sUART_serialRxIn);

IOP_N : IOPort
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(7 downto 0),
				dataRegWEn				=> ioP_N_dataRegWEn,
				dataRegOut				=> ioP_N_dataRegOut,
				dataOutEnWEn			=> ioP_N_dataOutEnWEn,
				dataOutEnOut			=> ioP_N_dataOutEnOut,
				ioPortDIn				=> ioP_N_dIn,
				ioPortOutEn				=> ioP_N_OutEn,
				ioPortDOut				=> ioP_N_dOut);

IOP_E : IOPort
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(7 downto 0),
				dataRegWEn				=> ioP_E_dataRegWEn,
				dataRegOut				=> ioP_E_dataRegOut,
				dataOutEnWEn			=> ioP_E_dataOutEnWEn,
				dataOutEnOut			=> ioP_E_dataOutEnOut,
				ioPortDIn				=> ioP_E_dIn,
				ioPortOutEn				=> ioP_E_OutEn,
				ioPortDOut				=> ioP_E_dOut);

IOP_W : IOPort
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(7 downto 0),
				dataRegWEn				=> ioP_W_dataRegWEn,
				dataRegOut				=> ioP_W_dataRegOut,
				dataOutEnWEn			=> ioP_W_dataOutEnWEn,
				dataOutEnOut			=> ioP_W_dataOutEnOut,
				ioPortDIn				=> ioP_W_dIn,
				ioPortOutEn				=> ioP_W_OutEn,
				ioPortDOut				=> ioP_W_dOut);

FABIO_N : FabricIOPort
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(5 downto 0),
				dataRegWEn				=> fabIO_N_dataRegWEn,
				dataRegOut				=> fabIO_N_dataRegOut,
				ioPortDIn				=> fabIO_N_dIn,
				ioPortDOut				=> fabIO_N_dOut);

FABIO_S : FabricIOPort
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(5 downto 0),
				dataRegWEn				=> fabIO_S_dataRegWEn,
				dataRegOut				=> fabIO_S_dataRegOut,
				ioPortDIn				=> fabIO_S_dIn,
				ioPortDOut				=> fabIO_S_dOut);

FABIO_E : FabricIOPort
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(5 downto 0),
				dataRegWEn				=> fabIO_E_dataRegWEn,
				dataRegOut				=> fabIO_E_dataRegOut,
				ioPortDIn				=> fabIO_E_dIn,
				ioPortDOut				=> fabIO_E_dOut);

FABIO_W : FabricIOPort
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn(5 downto 0),
				dataRegWEn				=> fabIO_W_dataRegWEn,
				dataRegOut				=> fabIO_W_dataRegOut,
				ioPortDIn				=> fabIO_W_dIn,
				ioPortDOut				=> fabIO_W_dOut);

CAP_L : FabricCAP
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> cap_L_controlRegWEn,
				controlRegOut			=> cap_L_controlRegOut,
				dataRegWEn				=> cap_L_dataRegWEn,
				dataRegOut				=> cap_L_dataRegOut,
				cnfChainLockRequest	=> cap_L_cnfChainLockRequest,
				cnfChainLock			=> cap_L_cnfChainLock,
				cnfChainShiftEn		=> cap_L_cnfChainShiftEn,
				cnfChainLoadEn			=> cap_L_cnfChainLoadEn,
				cnfChainReadBackEn	=> cap_L_cnfChainReadBackEn,
				cnfChainOut				=> cap_L_cnfChainOut,
				cnfChainIn				=> cap_L_cnfChainIn,
				selChainLockRequest	=> cap_L_selChainLockRequest,
				selChainLock			=> cap_L_selChainLock,
				selChainShiftEn		=> cap_L_selChainShiftEn,
				selChainFeedbackEn	=> cap_L_selChainFeedbackEn,
				selChainOut				=> cap_L_selChainOut,
				selChainIn				=> cap_L_selChainIn);

CAP_R : FabricCAP
port map (	clk						=> clk,
				enable					=> enable,
				nReset					=> nReset,
				dIn						=> portADIn,
				controlRegWEn			=> cap_R_controlRegWEn,
				controlRegOut			=> cap_R_controlRegOut,
				dataRegWEn				=> cap_R_dataRegWEn,
				dataRegOut				=> cap_R_dataRegOut,
				cnfChainLockRequest	=> cap_R_cnfChainLockRequest,
				cnfChainLock			=> cap_R_cnfChainLock,
				cnfChainShiftEn		=> cap_R_cnfChainShiftEn,
				cnfChainLoadEn			=> cap_R_cnfChainLoadEn,
				cnfChainReadBackEn	=> cap_R_cnfChainReadBackEn,
				cnfChainOut				=> cap_R_cnfChainOut,
				cnfChainIn				=> cap_R_cnfChainIn,
				selChainLockRequest	=> cap_R_selChainLockRequest,
				selChainLock			=> cap_R_selChainLock,
				selChainShiftEn		=> cap_R_selChainShiftEn,
				selChainFeedbackEn	=> cap_R_selChainFeedbackEn,
				selChainOut				=> cap_R_selChainOut,
				selChainIn				=> cap_R_selChainIn);


-- Interrupts -------------------------------------------------------------------------------------------------
-- Connect Interrupt sources to InterruptController
---------------------------------------------------------------------------------------------------------------
intC_interruptsIn(0)		<= intC_extIRQsIn(0);
intC_interruptsIn(1)		<= intC_extIRQsIn(1);
intC_interruptsIn(2)		<= intC_extIRQsIn(2);
intC_interruptsIn(3)		<= intC_extIRQsIn(3);
intC_interruptsIn(4)		<= intC_extIRQsIn(4);
intC_interruptsIn(5)		<= intC_extIRQsIn(5);
intC_interruptsIn(6)		<= intC_extIRQsIn(6);
intC_interruptsIn(7)		<= intC_extIRQsIn(7);
intC_interruptsIn(8)		<= cnt_0_overflowIF;
intC_interruptsIn(9)		<= cnt_0_match0IF;
intC_interruptsIn(10)	<= cnt_0_match1IF;
intC_interruptsIn(11)	<= cnt_1_overflowIF;
intC_interruptsIn(12)	<= cnt_1_match0IF;
intC_interruptsIn(13)	<= cnt_1_match1IF;
intC_interruptsIn(14)	<='0';
intC_interruptsIn(15)	<='0';

intC_interruptsIn(16)	<= sLnk_N_newDataIF;
intC_interruptsIn(17)	<= sLnk_N_rxFifoFullIF;
intC_interruptsIn(18)	<= sLnk_N_txFreeIF;
intC_interruptsIn(19)	<= sLnk_E_newDataIF;
intC_interruptsIn(20)	<= sLnk_E_rxFifoFullIF;
intC_interruptsIn(21)	<= sLnk_E_txFreeIF;
intC_interruptsIn(22)	<= sLnk_S_newDataIF;
intC_interruptsIn(23)	<= sLnk_S_rxFifoFullIF;
intC_interruptsIn(24)	<= sLnk_S_txFreeIF;
intC_interruptsIn(25)	<= sLnk_W_newDataIF;
intC_interruptsIn(26)	<= sLnk_W_rxFifoFullIF;
intC_interruptsIn(27)	<= sLnk_W_txFreeIF;
intC_interruptsIn(28)	<= wdt_timeoutIF;
intC_interruptsIn(29)	<= sUART_newDataIF;
intC_interruptsIn(30)	<= sUART_rxFifoFullIF;
intC_interruptsIn(31)	<= sUART_txFreeIF;


-- Write Control ----------------------------------------------------------------------------------------------
-- Write Enable control
---------------------------------------------------------------------------------------------------------------
WriteControl : process(portAAdd, portAWEn)

	variable regNumA : SNAPTYPE_REG_NUM;

begin

	wReg_portAWEn				<= '0';

	pc_pcWEn						<= '0';

	fReg_flagReg0WEn			<= '0';
	fReg_flagReg1WEn			<= '0';

	intC_flagsReg0WEn			<= '0';
	intC_flagsReg1WEn			<= '0';
	intC_enableReg0WEn		<= '0';
	intC_enableReg1WEn		<= '0';
	intC_intClr0WEn			<= '0';
	intC_intClr1WEn			<= '0';
	intC_intSet0WEn			<= '0';
	intC_intSet1WEn			<= '0';

	sLnk_N_controlRegWEn		<= '0';
	sLnk_N_dataRegWEn			<= '0';
	sLnk_E_controlRegWEn		<= '0';
	sLnk_E_dataRegWEn			<= '0';
	sLnk_S_controlRegWEn		<= '0';
	sLnk_S_dataRegWEn			<= '0';
	sLnk_W_controlRegWEn		<= '0';
	sLnk_W_dataRegWEn			<= '0';

	cnt_0_controlRegWEn		<= '0';
	cnt_0_counterWEn			<= '0';
	cnt_0_compareReg0WEn		<= '0';
	cnt_0_compareReg1WEn		<= '0';
	cnt_1_controlRegWEn		<= '0';
	cnt_1_counterWEn			<= '0';
	cnt_1_compareReg0WEn		<= '0';
	cnt_1_compareReg1WEn		<= '0';

	wdt_controlRegWEn			<= '0';
	wdt_timerWEn				<= '0';

	sUART_controlRegWEn		<= '0';
	sUART_dataRegWEn			<= '0';
	sUART_baudRegWEn			<= '0';

	ioP_N_dataRegWEn			<= '0';
	ioP_N_dataOutEnWEn		<= '0';
	ioP_E_dataRegWEn			<= '0';
	ioP_E_dataOutEnWEn		<= '0';
	ioP_W_dataRegWEn			<= '0';
	ioP_W_dataOutEnWEn		<= '0';

	fabIO_N_dataRegWEn		<= '0';
	fabIO_S_dataRegWEn		<= '0';
	fabIO_E_dataRegWEn		<= '0';
	fabIO_W_dataRegWEn		<= '0';

	cap_L_controlRegWEn		<= '0';
	cap_L_dataRegWEn			<= '0';
	cap_R_controlRegWEn		<= '0';
	cap_R_dataRegWEn			<= '0';

	if (portAWEn = '1') then

		regNumA := to_integer( unsigned(portAAdd) );

		case regNumA is

			-- Working Registers
			when SNAP_REG_R00 to SNAP_REG_R31 =>	wReg_portAWEn	<= '1';

			-- Program Counter
			when SNAP_REG_PC_PC				=>	pc_pcWEn					<= '1';

			-- Flags Registers
			when SNAP_REG_FREG_FLAGS0		=>	fReg_flagReg0WEn		<= '1';
			when SNAP_REG_FREG_FLAGS1		=>	fReg_flagReg1WEn		<= '1';

			-- Interrupt Controller
			when SNAP_REG_INTC_FLAGS_0		=>	intC_flagsReg0WEn		<= '1';
			when SNAP_REG_INTC_FLAGS_1		=>	intC_flagsReg1WEn		<= '1';
			when SNAP_REG_INTC_ENABLE_0	=>	intC_enableReg0WEn	<= '1';
			when SNAP_REG_INTC_ENABLE_1	=>	intC_enableReg1WEn	<= '1';
			when SNAP_REG_INTC_CLEAR_0		=>	intC_intClr0WEn		<= '1';
			when SNAP_REG_INTC_CLEAR_1		=>	intC_intClr1WEn		<= '1';
			when SNAP_REG_INTC_SET_0		=>	intC_intSet0WEn		<= '1';
			when SNAP_REG_INTC_SET_1		=>	intC_intSet1WEn		<= '1';

			-- SnapLink North
			when SNAP_REG_SLINK_N_CTRL		=>	sLnk_N_controlRegWEn	<= '1';

			when SNAP_REG_SLINK_N_DATA		=>	sLnk_N_dataRegWEn		<= '1';

			-- SnapLink East
			when SNAP_REG_SLINK_E_CTRL		=>	sLnk_E_controlRegWEn	<= '1';

			when SNAP_REG_SLINK_E_DATA		=>	sLnk_E_dataRegWEn		<= '1';

			-- SnapLink South
			when SNAP_REG_SLINK_S_CTRL		=>	sLnk_S_controlRegWEn	<= '1';
			when SNAP_REG_SLINK_S_DATA		=>	sLnk_S_dataRegWEn		<= '1';

			-- SnapLink West
			when SNAP_REG_SLINK_W_CTRL		=>	sLnk_W_controlRegWEn	<= '1';
			when SNAP_REG_SLINK_W_DATA		=>	sLnk_W_dataRegWEn		<= '1';

			-- Counter 0
			when SNAP_REG_CNT_0_CTRL		=>	cnt_0_controlRegWEn	<= '1';
			when SNAP_REG_CNT_0_COUNTER	=>	cnt_0_counterWEn		<= '1';
			when SNAP_REG_CNT_0_COMP0		=>	cnt_0_compareReg0WEn	<= '1';
			when SNAP_REG_CNT_0_COMP1		=>	cnt_0_compareReg1WEn	<= '1';

			-- Counter 1
			when SNAP_REG_CNT_1_CTRL		=>	cnt_1_controlRegWEn	<= '1';
			when SNAP_REG_CNT_1_COUNTER	=>	cnt_1_counterWEn		<= '1';
			when SNAP_REG_CNT_1_COMP0		=>	cnt_1_compareReg0WEn	<= '1';
			when SNAP_REG_CNT_1_COMP1		=>	cnt_1_compareReg1WEn	<= '1';

			-- Watch Dog Timer
			when SNAP_REG_WDT_CTRL			=>	wdt_controlRegWEn		<= '1';
			when SNAP_REG_WDT_TIMER			=>	wdt_timerWEn			<= '1';

			-- System UART
			when SNAP_REG_SUART_CTRL		=>	sUART_controlRegWEn	<= '1';
			when SNAP_REG_SUART_DATA		=>	sUART_dataRegWEn		<= '1';
			when SNAP_REG_SUART_BAUD		=>	sUART_baudRegWEn		<= '1';

			-- IO Ports
			when SNAP_REG_IOP_N_DATA		=> ioP_N_dataRegWEn		<= '1';
			when SNAP_REG_IOP_N_OUTEN		=> ioP_N_dataOutEnWEn	<= '1';
			when SNAP_REG_IOP_E_DATA		=> ioP_E_dataRegWEn		<= '1';
			when SNAP_REG_IOP_E_OUTEN		=> ioP_E_dataOutEnWEn	<= '1';
			when SNAP_REG_IOP_W_DATA		=> ioP_W_dataRegWEn		<= '1';
			when SNAP_REG_IOP_W_OUTEN		=> ioP_W_dataOutEnWEn	<= '1';

			-- Fabric IO Port
			when SNAP_REG_FABIO_N_DATA		=>	fabIO_N_dataRegWEn	<= '1';
			when SNAP_REG_FABIO_S_DATA		=>	fabIO_S_dataRegWEn	<= '1';
			when SNAP_REG_FABIO_E_DATA		=>	fabIO_E_dataRegWEn	<= '1';
			when SNAP_REG_FABIO_W_DATA		=>	fabIO_W_dataRegWEn	<= '1';

			-- Fabric CAP Logic
			when SNAP_REG_CAP_L_CTRL		=>	cap_L_controlRegWEn	<= '1';
			when SNAP_REG_CAP_L_DATA		=>	cap_L_dataRegWEn		<= '1';

			-- Fabric CAP Route
			when SNAP_REG_CAP_R_CTRL		=>	cap_R_controlRegWEn	<= '1';
			when SNAP_REG_CAP_R_DATA		=>	cap_R_dataRegWEn		<= '1';

			when others =>
				null;

		end case;

	end if;

end process;

-- Read Control ----------------------------------------------------------------------------------------------
-- Read Enable control
---------------------------------------------------------------------------------------------------------------
ReadControl : process(portBAdd, portBREn)

	variable regNumB : SNAPTYPE_REG_NUM;

begin

	sLnk_N_dataRegREn <= '0';
	sLnk_E_dataRegREn <= '0';
	sLnk_S_dataRegREn <= '0';
	sLnk_W_dataRegREn <= '0';

	sUART_dataRegREn	<= '0';

	if (portBREn = '1') then

		regNumB := to_integer( unsigned(portBAdd) );

		case regNumB is

			-- Snap Link North
			when SNAP_REG_SLINK_N_DATA	=>	sLnk_N_dataRegREn	<= '1';

			-- Snap Link East
			when SNAP_REG_SLINK_E_DATA	=>	sLnk_E_dataRegREn	<= '1';

			-- Snap Link South
			when SNAP_REG_SLINK_S_DATA	=>	sLnk_S_dataRegREn	<= '1';

			-- Snap Link West
			when SNAP_REG_SLINK_W_DATA	=>	sLnk_W_dataRegREn	<= '1';

			-- Snap Link West
			when SNAP_REG_SUART_DATA	=>	sUART_dataRegREn	<= '1';

			-- Others
			when others =>	null;

		end case;

	end if;

end process;

-- PortADecode ------------------------------------------------------------------------------------------------
-- Port A Address Decoder, output selector
---------------------------------------------------------------------------------------------------------------
PortADecode : process(	portAAdd, portAWEn,
								wReg_portADOut,
								pc_pcOut_sig, pc_braPCOut, pc_isrPCOut,
								fReg_flagReg0Out, fReg_flagReg1Out,
								intC_flagsReg0Out, intC_flagsReg1Out, intC_enableReg0Out, intC_enableReg1Out,
								sLnk_N_controlRegOut, sLnk_N_dataRegOut, sLnk_E_controlRegOut, sLnk_E_dataRegOut,
								sLnk_S_controlRegOut, sLnk_S_dataRegOut, sLnk_W_controlRegOut, sLnk_W_dataRegOut,
								cnt_0_controlRegOut, cnt_0_counterOut, cnt_0_compareReg0Out, cnt_0_compareReg1Out,
								cnt_1_controlRegOut, cnt_1_counterOut, cnt_1_compareReg0Out, cnt_1_compareReg1Out,
								wdt_controlRegOut, wdt_timerOut,
								sUART_controlRegOut, sUART_dataRegOut, sUART_baudRegOut,
								rng_randomDataIn,
								ioP_N_dataRegOut, ioP_N_dataOutEnOut,
								ioP_E_dataRegOut, ioP_E_dataOutEnOut,
								ioP_W_dataRegOut, ioP_W_dataOutEnOut,
								fabIO_N_dataRegOut, fabIO_S_dataRegOut, fabIO_E_dataRegOut, fabIO_W_dataRegOut,
								cap_L_controlRegOut, cap_L_dataRegOut, cap_R_controlRegOut, cap_R_dataRegOut)

	variable regNumA	: SNAPTYPE_REG_NUM;
	variable dOut		: std_logic_vector(15 downto 0);

begin

	regNumA	:= to_integer( unsigned(portAAdd) );
	dOut		:= (others => '0');

	case regNumA is

		-- Working Registers
		when SNAP_REG_R00 to SNAP_REG_R31 => dOut := wReg_portADOut;

		-- Program Counter
		when SNAP_REG_PC_PC				=>	dOut(SNAPTYPE_MEM_ADD'range) := pc_pcOut_sig;
		when SNAP_REG_PC_BRA_PC			=>	dOut(SNAPTYPE_MEM_ADD'range) := pc_braPCOut;
		when SNAP_REG_PC_ISR_PC			=>	dOut(SNAPTYPE_MEM_ADD'range) := pc_isrPCOut;

		-- Flags Registers
		when SNAP_REG_FREG_FLAGS0		=>	dOut := fReg_flagReg0Out;
		when SNAP_REG_FREG_FLAGS1		=>	dOut := fReg_flagReg1Out;

		-- Interrupt Controller
		when SNAP_REG_INTC_FLAGS_0		=>	dOut := intC_flagsReg0Out;
		when SNAP_REG_INTC_FLAGS_1		=>	dOut := intC_flagsReg1Out;
		when SNAP_REG_INTC_ENABLE_0	=>	dOut := intC_enableReg0Out;
		when SNAP_REG_INTC_ENABLE_1	=>	dOut := intC_enableReg1Out;

		-- SnapLink North
		when SNAP_REG_SLINK_N_CTRL		=>	dOut := sLnk_N_controlRegOut;
		when SNAP_REG_SLINK_N_DATA		=>	dOut := sLnk_N_dataRegOut;

		-- SnapLink East
		when SNAP_REG_SLINK_E_CTRL		=>	dOut := sLnk_E_controlRegOut;
		when SNAP_REG_SLINK_E_DATA		=>	dOut := sLnk_E_dataRegOut;

		-- SnapLink South
		when SNAP_REG_SLINK_S_CTRL		=>	dOut := sLnk_S_controlRegOut;
		when SNAP_REG_SLINK_S_DATA		=>	dOut := sLnk_S_dataRegOut;

		-- SnapLink West
		when SNAP_REG_SLINK_W_CTRL		=>	dOut := sLnk_W_controlRegOut;
		when SNAP_REG_SLINK_W_DATA		=>	dOut := sLnk_W_dataRegOut;

		-- Counter 0
		when SNAP_REG_CNT_0_CTRL		=>	dOut := cnt_0_controlRegOut;
		when SNAP_REG_CNT_0_COUNTER	=>	dOut := cnt_0_counterOut;
		when SNAP_REG_CNT_0_COMP0		=>	dOut := cnt_0_compareReg0Out;
		when SNAP_REG_CNT_0_COMP1		=>	dOut := cnt_0_compareReg1Out;

		-- Counter 1
		when SNAP_REG_CNT_1_CTRL		=>	dOut := cnt_1_controlRegOut;
		when SNAP_REG_CNT_1_COUNTER	=>	dOut := cnt_1_counterOut;
		when SNAP_REG_CNT_1_COMP0		=>	dOut := cnt_1_compareReg0Out;
		when SNAP_REG_CNT_1_COMP1		=>	dOut := cnt_1_compareReg1Out;

		-- Watch Dog Timer
		when SNAP_REG_WDT_CTRL			=>	dOut := wdt_controlRegOut;
		when SNAP_REG_WDT_TIMER			=>	dOut := wdt_timerOut;

		-- System UART
		when SNAP_REG_SUART_CTRL		=>	dOut := sUART_controlRegOut;
		when SNAP_REG_SUART_DATA		=>	dOut := sUART_dataRegOut;
		when SNAP_REG_SUART_BAUD		=>	dOut := sUART_baudRegOut;

		-- Random Number Generator
		when SNAP_REG_RNG_DATA			=>	dOut := rng_randomDataIn;

		-- IO Ports
		when SNAP_REG_IOP_N_DATA		=> dOut(7 downto 0) := ioP_N_dataRegOut;
		when SNAP_REG_IOP_N_OUTEN		=> dOut(7 downto 0) := ioP_N_dataOutEnOut;
		when SNAP_REG_IOP_E_DATA		=> dOut(7 downto 0) := ioP_E_dataRegOut;
		when SNAP_REG_IOP_E_OUTEN		=> dOut(7 downto 0) := ioP_E_dataOutEnOut;
		when SNAP_REG_IOP_W_DATA		=> dOut(7 downto 0) := ioP_W_dataRegOut;
		when SNAP_REG_IOP_W_OUTEN		=> dOut(7 downto 0) := ioP_W_dataOutEnOut;

		-- Fabric IO Port
		when SNAP_REG_FABIO_N_DATA		=>	dOut(5 downto 0) := fabIO_N_dataRegOut;
		when SNAP_REG_FABIO_S_DATA		=>	dOut(5 downto 0) := fabIO_S_dataRegOut;
		when SNAP_REG_FABIO_E_DATA		=>	dOut(5 downto 0) := fabIO_E_dataRegOut;
		when SNAP_REG_FABIO_W_DATA		=>	dOut(5 downto 0) := fabIO_W_dataRegOut;

		-- Fabric CAP Logic
		when SNAP_REG_CAP_L_CTRL		=>	dOut := cap_L_controlRegOut;
		when SNAP_REG_CAP_L_DATA		=>	dOut := cap_L_dataRegOut;

		-- Fabric CAP Route
		when SNAP_REG_CAP_R_CTRL		=>	dOut := cap_R_controlRegOut;
		when SNAP_REG_CAP_R_DATA		=>	dOut := cap_R_dataRegOut;

		-- Others
		when others							=>	dOut := b"0000_0000_0000_0000";

	end case;

	portADOut <= dOut;

end process;


-- PortBDecode ------------------------------------------------------------------------------------------------
-- Port B Address Decoder, output selector
---------------------------------------------------------------------------------------------------------------
PortBDecode : process(	portBAdd,
								wReg_portBDOut,
								pc_pcOut_sig, pc_braPCOut, pc_isrPCOut,
								fReg_flagReg0Out, fReg_flagReg1Out,
								intC_flagsReg0Out, intC_flagsReg1Out, intC_enableReg0Out, intC_enableReg1Out,
								sLnk_N_controlRegOut, sLnk_N_dataRegOut, sLnk_E_controlRegOut, sLnk_E_dataRegOut,
								sLnk_S_controlRegOut, sLnk_S_dataRegOut, sLnk_W_controlRegOut, sLnk_W_dataRegOut,
								cnt_0_controlRegOut, cnt_0_counterOut, cnt_0_compareReg0Out, cnt_0_compareReg1Out,
								cnt_1_controlRegOut, cnt_1_counterOut, cnt_1_compareReg0Out, cnt_1_compareReg1Out,
								wdt_controlRegOut, wdt_timerOut,
								sUART_controlRegOut, sUART_dataRegOut, sUART_baudRegOut,
								rng_randomDataIn,
								ioP_N_dataRegOut, ioP_N_dataOutEnOut,
								ioP_E_dataRegOut, ioP_E_dataOutEnOut,
								ioP_W_dataRegOut, ioP_W_dataOutEnOut,
								fabIO_N_dataRegOut, fabIO_S_dataRegOut, fabIO_E_dataRegOut, fabIO_W_dataRegOut,
								cap_L_controlRegOut, cap_L_dataRegOut, cap_R_controlRegOut, cap_R_dataRegOut)

	variable regNumB	: SNAPTYPE_REG_NUM;
	variable dOut		: std_logic_vector(15 downto 0);

begin

	regNumB	:= to_integer( unsigned(portBAdd) );
	dOut		:= (others => '0');

	case regNumB is

		-- Working Registers
		when SNAP_REG_R00 to SNAP_REG_R31 => dOut := wReg_portBDOut;

		-- Program Counter
		when SNAP_REG_PC_PC				=>	dOut(SNAPTYPE_MEM_ADD'range) := pc_pcOut_sig;
		when SNAP_REG_PC_BRA_PC			=>	dOut(SNAPTYPE_MEM_ADD'range) := pc_braPCOut;
		when SNAP_REG_PC_ISR_PC			=>	dOut(SNAPTYPE_MEM_ADD'range) := pc_isrPCOut;

		-- Flags Registers
		when SNAP_REG_FREG_FLAGS0		=>	dOut := fReg_flagReg0Out;
		when SNAP_REG_FREG_FLAGS1		=>	dOut := fReg_flagReg1Out;

			-- Interrupt Controller
		when SNAP_REG_INTC_FLAGS_0		=>	dOut := intC_flagsReg0Out;
		when SNAP_REG_INTC_FLAGS_1		=>	dOut := intC_flagsReg1Out;
		when SNAP_REG_INTC_ENABLE_0	=>	dOut := intC_enableReg0Out;
		when SNAP_REG_INTC_ENABLE_1	=>	dOut := intC_enableReg1Out;

		-- SnapLink North
		when SNAP_REG_SLINK_N_CTRL		=>	dOut := sLnk_N_controlRegOut;
		when SNAP_REG_SLINK_N_DATA		=>	dOut := sLnk_N_dataRegOut;

		-- SnapLink East
		when SNAP_REG_SLINK_E_CTRL		=>	dOut := sLnk_E_controlRegOut;
		when SNAP_REG_SLINK_E_DATA		=>	dOut := sLnk_E_dataRegOut;

		-- SnapLink South
		when SNAP_REG_SLINK_S_CTRL		=>	dOut := sLnk_S_controlRegOut;
		when SNAP_REG_SLINK_S_DATA		=>	dOut := sLnk_S_dataRegOut;

		-- SnapLink West
		when SNAP_REG_SLINK_W_CTRL		=>	dOut := sLnk_W_controlRegOut;
		when SNAP_REG_SLINK_W_DATA		=>	dOut := sLnk_W_dataRegOut;

		-- Counter 0
		when SNAP_REG_CNT_0_CTRL		=>	dOut := cnt_0_controlRegOut;
		when SNAP_REG_CNT_0_COUNTER	=>	dOut := cnt_0_counterOut;
		when SNAP_REG_CNT_0_COMP0		=>	dOut := cnt_0_compareReg0Out;
		when SNAP_REG_CNT_0_COMP1		=>	dOut := cnt_0_compareReg1Out;

		-- Counter 1
		when SNAP_REG_CNT_1_CTRL		=>	dOut := cnt_1_controlRegOut;
		when SNAP_REG_CNT_1_COUNTER	=>	dOut := cnt_1_counterOut;
		when SNAP_REG_CNT_1_COMP0		=>	dOut := cnt_1_compareReg0Out;
		when SNAP_REG_CNT_1_COMP1		=>	dOut := cnt_1_compareReg1Out;

		-- Watch Dog Timer
		when SNAP_REG_WDT_CTRL			=>	dOut := wdt_controlRegOut;
		when SNAP_REG_WDT_TIMER			=>	dOut := wdt_timerOut;

		-- System UART
		when SNAP_REG_SUART_CTRL		=>	dOut := sUART_controlRegOut;
		when SNAP_REG_SUART_DATA		=>	dOut := sUART_dataRegOut;
		when SNAP_REG_SUART_BAUD		=>	dOut := sUART_baudRegOut;

		-- Random Number Generator
		when SNAP_REG_RNG_DATA			=>	dOut := rng_randomDataIn;

		-- IO Ports
		when SNAP_REG_IOP_N_DATA		=> dOut(7 downto 0) := ioP_N_dataRegOut;
		when SNAP_REG_IOP_N_OUTEN		=> dOut(7 downto 0) := ioP_N_dataOutEnOut;
		when SNAP_REG_IOP_E_DATA		=> dOut(7 downto 0) := ioP_E_dataRegOut;
		when SNAP_REG_IOP_E_OUTEN		=> dOut(7 downto 0) := ioP_E_dataOutEnOut;
		when SNAP_REG_IOP_W_DATA		=> dOut(7 downto 0) := ioP_W_dataRegOut;
		when SNAP_REG_IOP_W_OUTEN		=> dOut(7 downto 0) := ioP_W_dataOutEnOut;

		-- Fabric IO Port
		when SNAP_REG_FABIO_N_DATA		=>	dOut(5 downto 0) := fabIO_N_dataRegOut;
		when SNAP_REG_FABIO_S_DATA		=>	dOut(5 downto 0) := fabIO_S_dataRegOut;
		when SNAP_REG_FABIO_E_DATA		=>	dOut(5 downto 0) := fabIO_E_dataRegOut;
		when SNAP_REG_FABIO_W_DATA		=>	dOut(5 downto 0) := fabIO_W_dataRegOut;

		-- Fabric CAP Logic
		when SNAP_REG_CAP_L_CTRL		=>	dOut := cap_L_controlRegOut;
		when SNAP_REG_CAP_L_DATA		=>	dOut := cap_L_dataRegOut;

		-- Fabric CAP Route
		when SNAP_REG_CAP_R_CTRL		=>	dOut := cap_R_controlRegOut;
		when SNAP_REG_CAP_R_DATA		=>	dOut := cap_R_dataRegOut;

		-- Others
		when others							=>	dOut := b"0000_0000_0000_0000";

	end case;

	portBDOut <= dOut;

end process;

end General;
